% *************************************************************************
% ******************** Friedman-GRA Data Set Generator ********************
%    A synthetic regression problem using a modified Friedman's function
%                      with Global Recurring Abrupt drift
% *************************************************************************
% The Toolbox of Regression Data Sets for Concept Drifts
% Version 1.0 Nov/10/2014
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com or symonesoares@isr.uc.pt
%
% Homepage: http://sites.google.com/site/symonesoares/
%
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. An On-line Weighted Ensemble 
%        of Regressor Models to Handle Concept Drifts. Engineering  
%        Applications of Artificial Intelligence, v. 37, p. 392-406, 2015. 
%     url: http://www.sciencedirect.com/science/article/pii/S09521976140024
%
% Reference: IKONOMOVSKA, E.; GAMA, J.; SEBASTIãO, R.; GJORGJEVIK, D.;
%            Regression Trees from Data Streams with Drift Detection. 
%            August 7-11 2005. In: Discovery Science, Vol. 5808 of Lecture
%            Notes in Computer Science, 2009, pp. 121–135.
%
% *************************************************************************
% *************************************************************************

clear; clc;
N = 2000; % Define the number of samples
n   = floor(N / 4);  

% First Generating whole data set
X = rand(N, 10);
Y = 10 * sin(pi * X(:,1) .* X(:,2)) + 20 * (X(:,3) - 0.5).^2 + 10 * X(:,4) + 5 * X(:,5) + randn(N,1);
DriftedSamples = [];
Yreal = Y;

% I. change point
init = n*2 + 1;
for i = init:n*3
    Y(i,1) = 10 * sin(pi * X(i,4) * X(i,5)) + 20 * (X(i,2) - 0.5)^2 + 10 * X(i,1) + 5 * X(i,3) + randn(1,1);
    DriftedSamples = [DriftedSamples i];
end

% II. change point
% Reocurring concept (original function)

input = X;
output = Y;

save dSetFriedDriftII.mat DriftedSamples input output Yreal