% *************************************************************************
% ******************** Friedman-LA Data Set Generator *********************
%    A synthetic regression problem using a modified Friedman's function
%                      with Local and Abrupt drift
% *************************************************************************
% The Toolbox of Regression Data Sets for Concept Drifts
% Version 1.0 Nov/10/2014
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com or symonesoares@isr.uc.pt
%
% Homepage: http://sites.google.com/site/symonesoares/
%
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. An On-line Weighted Ensemble 
%        of Regressor Models to Handle Concept Drifts. Engineering  
%        Applications of Artificial Intelligence, v. 37, p. 392-406, 2015. 
%     url: http://www.sciencedirect.com/science/article/pii/S09521976140024
%
% Reference: IKONOMOVSKA, E.; GAMA, J.; SEBASTIãO, R.; GJORGJEVIK, D.;
%            Regression Trees from Data Streams with Drift Detection. 
%            August 7-11 2005. In: Discovery Science, Vol. 5808 of Lecture
%            Notes in Computer Science, 2009, pp. 121–135.
%
% *************************************************************************
% *************************************************************************

clear; clc;
N = 2000; % Define the number of samples
n   = floor(N / 4);  

% First Generating whole data set
X = rand(N, 10);
Y = 10 * sin(pi * X(:,1) .* X(:,2)) + 20 * (X(:,3) - 0.5).^2 + 10 * X(:,4) + 5 * X(:,5) + randn(N,1);
DriftedSamples = [];
Yreal = Y;

% I. change point
init = n + 1;
for i = init:(n*2)
    if (X(i,2) < 0.3) && (X(i,3) < 0.3) && (X(i,4) > 0.7) && (X(i,5) < 0.3)
      Y(i,1) = 10 * X(i,1) * X(i,2) + 20 * (X(i,3) - 0.5) + 10 * X(i,4) + 5 * X(i,5) + randn(1,1);
      DriftedSamples = [DriftedSamples i];
    end
end

% II. change point
init = n * 2 + 1;
for i = init:(n*3)
    if (X(i,2) > 0.7) && (X(i,3) > 0.7) && (X(i,4) < 0.3) && (X(i,5) > 0.7)
      Y(i,1) = 10 * cos(X(i,1) * X(i,2)) + 20 * (X(i,3) - 0.5) + exp(X(i,4)) + 5 * (X(i,5)^2) + randn(1,1);
      DriftedSamples = [DriftedSamples i];
    end
end

% III. change point
init = n * 3 + 1;
for i = init:N
    if (X(i,2) < 0.3) && (X(i,3) < 0.3) && (X(i,5) < 0.3) 
      Y(i,1) = 10 * X(i,1) * X(i,2) + 20 * (X(i,3) - 0.5) + 10 * X(i,4) + 5 * X(i,5) + randn(1,1);
      DriftedSamples = [DriftedSamples i];
    end
    if (X(i,2) > 0.7) && (X(i,3) > 0.7) && (X(i,5) > 0.7)
      Y(i,1) = 10 * cos(X(i,1) * X(i,2)) + 20 * (X(i,3) - 0.5) + exp(X(i,4)) + 5 * (X(i,5)^2) + randn(1,1);
      DriftedSamples = [DriftedSamples i];
    end
end

input = X;
output = Y;

save dSetFriedDriftI.mat DriftedSamples input output Yreal